package javax.swing;

import javax.accessibility.AccessibleKeyBinding;
import javax.swing.text.Element;
import javax.swing.text.AttributeSet;
import java.text.BreakIterator;
import javax.swing.text.Document;
import javax.swing.text.StyledDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.Position;
import java.awt.geom.Rectangle2D;
import java.awt.Point;
import javax.swing.text.View;
import java.util.Enumeration;
import javax.accessibility.AccessibleRelation;
import javax.accessibility.AccessibleRelationSet;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.Accessible;
import javax.accessibility.AccessibleIcon;
import javax.accessibility.AccessibleExtendedComponent;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleValue;
import javax.accessibility.AccessibleAction;
import java.io.Serializable;
import java.beans.PropertyChangeEvent;
import javax.accessibility.AccessibleContext;
import java.awt.Image;
import javax.accessibility.AccessibleState;
import java.awt.event.ItemEvent;
import java.awt.event.ActionEvent;
import java.awt.LayoutManager;
import java.awt.Component;
import javax.swing.plaf.ButtonUI;
import java.awt.Graphics;
import java.beans.Transient;
import javax.swing.plaf.UIResource;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.beans.PropertyChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ItemListener;
import java.awt.event.ActionListener;
import javax.swing.event.ChangeListener;
import java.awt.Insets;
import java.awt.ItemSelectable;

public abstract class AbstractButton extends JComponent implements ItemSelectable, SwingConstants
{
    public static final String MODEL_CHANGED_PROPERTY = "model";
    public static final String TEXT_CHANGED_PROPERTY = "text";
    public static final String MNEMONIC_CHANGED_PROPERTY = "mnemonic";
    public static final String MARGIN_CHANGED_PROPERTY = "margin";
    public static final String VERTICAL_ALIGNMENT_CHANGED_PROPERTY = "verticalAlignment";
    public static final String HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY = "horizontalAlignment";
    public static final String VERTICAL_TEXT_POSITION_CHANGED_PROPERTY = "verticalTextPosition";
    public static final String HORIZONTAL_TEXT_POSITION_CHANGED_PROPERTY = "horizontalTextPosition";
    public static final String BORDER_PAINTED_CHANGED_PROPERTY = "borderPainted";
    public static final String FOCUS_PAINTED_CHANGED_PROPERTY = "focusPainted";
    public static final String ROLLOVER_ENABLED_CHANGED_PROPERTY = "rolloverEnabled";
    public static final String CONTENT_AREA_FILLED_CHANGED_PROPERTY = "contentAreaFilled";
    public static final String ICON_CHANGED_PROPERTY = "icon";
    public static final String PRESSED_ICON_CHANGED_PROPERTY = "pressedIcon";
    public static final String SELECTED_ICON_CHANGED_PROPERTY = "selectedIcon";
    public static final String ROLLOVER_ICON_CHANGED_PROPERTY = "rolloverIcon";
    public static final String ROLLOVER_SELECTED_ICON_CHANGED_PROPERTY = "rolloverSelectedIcon";
    public static final String DISABLED_ICON_CHANGED_PROPERTY = "disabledIcon";
    public static final String DISABLED_SELECTED_ICON_CHANGED_PROPERTY = "disabledSelectedIcon";
    protected ButtonModel model;
    private String text;
    private Insets margin;
    private Insets defaultMargin;
    private Icon defaultIcon;
    private Icon pressedIcon;
    private Icon disabledIcon;
    private Icon selectedIcon;
    private Icon disabledSelectedIcon;
    private Icon rolloverIcon;
    private Icon rolloverSelectedIcon;
    private boolean paintBorder;
    private boolean paintFocus;
    private boolean rolloverEnabled;
    private boolean contentAreaFilled;
    private int verticalAlignment;
    private int horizontalAlignment;
    private int verticalTextPosition;
    private int horizontalTextPosition;
    private int iconTextGap;
    private int mnemonic;
    private int mnemonicIndex;
    private long multiClickThreshhold;
    private boolean borderPaintedSet;
    private boolean rolloverEnabledSet;
    private boolean iconTextGapSet;
    private boolean contentAreaFilledSet;
    private boolean setLayout;
    boolean defaultCapable;
    private AbstractButton.Handler handler;
    protected ChangeListener changeListener;
    protected ActionListener actionListener;
    protected ItemListener itemListener;
    protected transient ChangeEvent changeEvent;
    private boolean hideActionText;
    private Action action;
    private PropertyChangeListener actionPropertyChangeListener;
    
    public AbstractButton() {
        this.model = null;
        this.text = "";
        this.margin = null;
        this.defaultMargin = null;
        this.defaultIcon = null;
        this.pressedIcon = null;
        this.disabledIcon = null;
        this.selectedIcon = null;
        this.disabledSelectedIcon = null;
        this.rolloverIcon = null;
        this.rolloverSelectedIcon = null;
        this.paintBorder = true;
        this.paintFocus = true;
        this.rolloverEnabled = false;
        this.contentAreaFilled = true;
        this.verticalAlignment = 0;
        this.horizontalAlignment = 0;
        this.verticalTextPosition = 0;
        this.horizontalTextPosition = 11;
        this.iconTextGap = 4;
        this.mnemonicIndex = -1;
        this.multiClickThreshhold = 0L;
        this.borderPaintedSet = false;
        this.rolloverEnabledSet = false;
        this.iconTextGapSet = false;
        this.contentAreaFilledSet = false;
        this.setLayout = false;
        this.defaultCapable = true;
        this.changeListener = null;
        this.actionListener = null;
        this.itemListener = null;
        this.hideActionText = false;
    }
    
    public void setHideActionText(boolean hideActionText) {
        if (hideActionText != this.hideActionText) {
            this.hideActionText = hideActionText;
            if (this.getAction() != null) {
                this.setTextFromAction(this.getAction(), false);
            }
            this.firePropertyChange("hideActionText", !hideActionText, hideActionText);
        }
    }
    
    public boolean getHideActionText() {
        return this.hideActionText;
    }
    
    public String getText() {
        return this.text;
    }
    
    public void setText(String s) {
        String text = this.text;
        this.firePropertyChange("text", text, this.text = s);
        this.updateDisplayedMnemonicIndex(s, this.getMnemonic());
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", text, s);
        }
        if (s == null || text == null || !s.equals(text)) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public boolean isSelected() {
        return this.model.isSelected();
    }
    
    public void setSelected(boolean selected) {
        this.isSelected();
        this.model.setSelected(selected);
    }
    
    public void doClick() {
        this.doClick(68);
    }
    
    public void doClick(int n) {
        Dimension size = this.getSize();
        this.model.setArmed(true);
        this.model.setPressed(true);
        this.paintImmediately(new Rectangle(0, 0, size.width, size.height));
        try {
            Thread.currentThread();
            Thread.sleep((long)n);
        }
        catch (InterruptedException ex) {}
        this.model.setPressed(false);
        this.model.setArmed(false);
    }
    
    public void setMargin(Insets defaultMargin) {
        if ((defaultMargin instanceof UIResource)) {
            this.defaultMargin = defaultMargin;
        }
        else if ((this.margin instanceof UIResource)) {
            this.defaultMargin = this.margin;
        }
        if (defaultMargin == null && this.defaultMargin != null) {
            defaultMargin = this.defaultMargin;
        }
        Insets margin = this.margin;
        this.firePropertyChange("margin", margin, this.margin = defaultMargin);
        if (margin == null || !margin.equals(defaultMargin)) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public Insets getMargin() {
        return (this.margin == null) ? null : ((Insets)this.margin.clone());
    }
    
    public Icon getIcon() {
        return this.defaultIcon;
    }
    
    public void setIcon(Icon newValue) {
        Icon defaultIcon = this.defaultIcon;
        this.defaultIcon = newValue;
        if (newValue != defaultIcon && this.disabledIcon instanceof UIResource) {
            this.disabledIcon = null;
        }
        this.firePropertyChange("icon", defaultIcon, newValue);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", defaultIcon, newValue);
        }
        if (newValue != defaultIcon) {
            if (newValue == null || defaultIcon == null || newValue.getIconWidth() != defaultIcon.getIconWidth() || newValue.getIconHeight() != defaultIcon.getIconHeight()) {
                this.revalidate();
            }
            this.repaint();
        }
    }
    
    public Icon getPressedIcon() {
        return this.pressedIcon;
    }
    
    public void setPressedIcon(Icon icon) {
        Icon pressedIcon = this.pressedIcon;
        this.firePropertyChange("pressedIcon", pressedIcon, this.pressedIcon = icon);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", pressedIcon, icon);
        }
        if (icon != pressedIcon && this.getModel().isPressed()) {
            this.repaint();
        }
    }
    
    public Icon getSelectedIcon() {
        return this.selectedIcon;
    }
    
    public void setSelectedIcon(Icon newValue) {
        Icon selectedIcon = this.selectedIcon;
        this.selectedIcon = newValue;
        if (newValue != selectedIcon && this.disabledSelectedIcon instanceof UIResource) {
            this.disabledSelectedIcon = null;
        }
        this.firePropertyChange("selectedIcon", selectedIcon, newValue);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", selectedIcon, newValue);
        }
        if (newValue != selectedIcon && this.isSelected()) {
            this.repaint();
        }
    }
    
    public Icon getRolloverIcon() {
        return this.rolloverIcon;
    }
    
    public void setRolloverIcon(Icon icon) {
        Icon rolloverIcon = this.rolloverIcon;
        this.firePropertyChange("rolloverIcon", rolloverIcon, this.rolloverIcon = icon);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", rolloverIcon, icon);
        }
        this.setRolloverEnabled(true);
        if (icon != rolloverIcon) {
            this.repaint();
        }
    }
    
    public Icon getRolloverSelectedIcon() {
        return this.rolloverSelectedIcon;
    }
    
    public void setRolloverSelectedIcon(Icon icon) {
        Icon rolloverSelectedIcon = this.rolloverSelectedIcon;
        this.firePropertyChange("rolloverSelectedIcon", rolloverSelectedIcon, this.rolloverSelectedIcon = icon);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", rolloverSelectedIcon, icon);
        }
        this.setRolloverEnabled(true);
        if (icon != rolloverSelectedIcon && this.isSelected()) {
            this.repaint();
        }
    }
    
    @Transient
    public Icon getDisabledIcon() {
        if (this.disabledIcon == null) {
            this.disabledIcon = UIManager.getLookAndFeel().getDisabledIcon(this, this.getIcon());
            if (this.disabledIcon != null) {
                this.firePropertyChange("disabledIcon", null, this.disabledIcon);
            }
        }
        return this.disabledIcon;
    }
    
    public void setDisabledIcon(Icon icon) {
        Icon disabledIcon = this.disabledIcon;
        this.firePropertyChange("disabledIcon", disabledIcon, this.disabledIcon = icon);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", disabledIcon, icon);
        }
        if (icon != disabledIcon && !this.isEnabled()) {
            this.repaint();
        }
    }
    
    public Icon getDisabledSelectedIcon() {
        if (this.disabledSelectedIcon == null) {
            if (this.selectedIcon != null) {
                this.disabledSelectedIcon = UIManager.getLookAndFeel().getDisabledSelectedIcon(this, this.getSelectedIcon());
            }
            else {
                return this.getDisabledIcon();
            }
        }
        return this.disabledSelectedIcon;
    }
    
    public void setDisabledSelectedIcon(Icon icon) {
        Icon disabledSelectedIcon = this.disabledSelectedIcon;
        this.firePropertyChange("disabledSelectedIcon", disabledSelectedIcon, this.disabledSelectedIcon = icon);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleVisibleData", disabledSelectedIcon, icon);
        }
        if (icon != disabledSelectedIcon) {
            if (icon == null || disabledSelectedIcon == null || icon.getIconWidth() != disabledSelectedIcon.getIconWidth() || icon.getIconHeight() != disabledSelectedIcon.getIconHeight()) {
                this.revalidate();
            }
            if (!this.isEnabled() && this.isSelected()) {
                this.repaint();
            }
        }
    }
    
    public int getVerticalAlignment() {
        return this.verticalAlignment;
    }
    
    public void setVerticalAlignment(int n) {
        if (n == this.verticalAlignment) {
            return;
        }
        this.firePropertyChange("verticalAlignment", this.verticalAlignment, this.verticalAlignment = this.checkVerticalKey(n, "verticalAlignment"));
        this.repaint();
        return;
    }
    
    public int getHorizontalAlignment() {
        return this.horizontalAlignment;
    }
    
    public void setHorizontalAlignment(int n) {
        if (n == this.horizontalAlignment) {
            return;
        }
        this.firePropertyChange("horizontalAlignment", this.horizontalAlignment, this.horizontalAlignment = this.checkHorizontalKey(n, "horizontalAlignment"));
        this.repaint();
        return;
    }
    
    public int getVerticalTextPosition() {
        return this.verticalTextPosition;
    }
    
    public void setVerticalTextPosition(int n) {
        if (n == this.verticalTextPosition) {
            return;
        }
        this.firePropertyChange("verticalTextPosition", this.verticalTextPosition, this.verticalTextPosition = this.checkVerticalKey(n, "verticalTextPosition"));
        this.revalidate();
        this.repaint();
        return;
    }
    
    public int getHorizontalTextPosition() {
        return this.horizontalTextPosition;
    }
    
    public void setHorizontalTextPosition(int n) {
        if (n == this.horizontalTextPosition) {
            return;
        }
        this.firePropertyChange("horizontalTextPosition", this.horizontalTextPosition, this.horizontalTextPosition = this.checkHorizontalKey(n, "horizontalTextPosition"));
        this.revalidate();
        this.repaint();
        return;
    }
    
    public int getIconTextGap() {
        return this.iconTextGap;
    }
    
    public void setIconTextGap(int iconTextGap) {
        int iconTextGap2 = this.iconTextGap;
        this.iconTextGap = iconTextGap;
        this.iconTextGapSet = true;
        this.firePropertyChange("iconTextGap", iconTextGap2, iconTextGap);
        if (iconTextGap != iconTextGap2) {
            this.revalidate();
            this.repaint();
        }
    }
    
    protected int checkHorizontalKey(int n, String s) {
        if (n == 2 || n == 0 || n == 4 || n == 10 || n == 11) {
            return n;
        }
        throw new IllegalArgumentException(s);
    }
    
    protected int checkVerticalKey(int n, String s) {
        if (n == 1 || n == 0 || n == 3) {
            return n;
        }
        throw new IllegalArgumentException(s);
    }
    
    @Override
    public void removeNotify() {
        super.removeNotify();
        if (this.isRolloverEnabled()) {
            this.getModel().setRollover(false);
        }
    }
    
    public void setActionCommand(String actionCommand) {
        this.getModel().setActionCommand(actionCommand);
    }
    
    public String getActionCommand() {
        String s = this.getModel().getActionCommand();
        if (s == null) {
            s = this.getText();
            return s;
        }
        return s;
    }
    
    public void setAction(Action action) {
        Action action2 = this.getAction();
        if (this.action == null || !this.action.equals(action)) {
            this.action = action;
            if (action2 != null) {
                this.removeActionListener(action2);
                action2.removePropertyChangeListener(this.actionPropertyChangeListener);
                this.actionPropertyChangeListener = null;
            }
            this.configurePropertiesFromAction(this.action);
            if (this.action != null) {
                if (!this.isListener(ActionListener.class, this.action)) {
                    this.addActionListener(this.action);
                }
                this.actionPropertyChangeListener = this.createActionPropertyChangeListener(this.action);
                this.action.addPropertyChangeListener(this.actionPropertyChangeListener);
            }
            this.firePropertyChange("action", action2, this.action);
        }
    }
    
    private boolean isListener(Class clazz, ActionListener actionListener) {
        boolean n = false;
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == clazz && listenerList[i + 1] == actionListener) {
                n = true;
            }
        }
        return n;
    }
    
    public Action getAction() {
        return this.action;
    }
    
    protected void configurePropertiesFromAction(Action action) {
        this.setMnemonicFromAction(action);
        this.setTextFromAction(action, false);
        AbstractAction.setToolTipTextFromAction(this, action);
        this.setIconFromAction(action);
        this.setActionCommandFromAction(action);
        AbstractAction.setEnabledFromAction(this, action);
        if (AbstractAction.hasSelectedKey(action) && this.shouldUpdateSelectedStateFromAction()) {
            this.setSelectedFromAction(action);
        }
        this.setDisplayedMnemonicIndexFromAction(action, false);
    }
    
    @Override
    void clientPropertyChanged(Object o, Object o2, Object o3) {
        if (o == "hideActionText") {
            int hideActionText = o3 instanceof Boolean && (boolean)o3;
            if ((this.getHideActionText() ? 1 : 0) != hideActionText) {
                this.setHideActionText((boolean)hideActionText);
            }
        }
    }
    
    boolean shouldUpdateSelectedStateFromAction() {
        return false;
    }
    
    protected void actionPropertyChanged(Action mnemonicFromAction, String s) {
        if (s == "Name") {
            this.setTextFromAction(mnemonicFromAction, true);
        }
        else if (s == "enabled") {
            AbstractAction.setEnabledFromAction(this, mnemonicFromAction);
        }
        else if (s == "ShortDescription") {
            AbstractAction.setToolTipTextFromAction(this, mnemonicFromAction);
        }
        else if (s == "SmallIcon") {
            this.smallIconChanged(mnemonicFromAction);
        }
        else if (s == "MnemonicKey") {
            this.setMnemonicFromAction(mnemonicFromAction);
        }
        else if (s == "ActionCommandKey") {
            this.setActionCommandFromAction(mnemonicFromAction);
        }
        else if (s == "SwingSelectedKey" && AbstractAction.hasSelectedKey(mnemonicFromAction) && this.shouldUpdateSelectedStateFromAction()) {
            this.setSelectedFromAction(mnemonicFromAction);
        }
        else if (s == "SwingDisplayedMnemonicIndexKey") {
            this.setDisplayedMnemonicIndexFromAction(mnemonicFromAction, true);
        }
        else if (s == "SwingLargeIconKey") {
            this.largeIconChanged(mnemonicFromAction);
        }
    }
    
    private void setDisplayedMnemonicIndexFromAction(Action action, boolean b) {
        Integer n = (action == null) ? null : ((Integer)action.getValue("SwingDisplayedMnemonicIndexKey"));
        if (b || n != null) {
            int intValue;
            if (n == null) {
                intValue = -1;
            }
            else {
                intValue = (int)n;
                String text = this.getText();
                if (text == null || intValue >= text.length()) {
                    intValue = -1;
                }
            }
            this.setDisplayedMnemonicIndex(intValue);
        }
    }
    
    private void setMnemonicFromAction(Action action) {
        Integer n = (action == null) ? null : ((Integer)action.getValue("MnemonicKey"));
        this.setMnemonic((n == null) ? 0 : ((int)n));
    }
    
    private void setTextFromAction(Action action, boolean b) {
        int hideActionText = this.getHideActionText();
        if (!b) {
            this.setText((action != null && hideActionText == 0) ? ((String)action.getValue("Name")) : null);
        }
        else if (hideActionText == 0) {
            this.setText((String)action.getValue("Name"));
        }
    }
    
    void setIconFromAction(Action action) {
        Icon icon = null;
        if (action != null) {
            icon = (Icon)action.getValue("SwingLargeIconKey");
            if (icon == null) {
                icon = (Icon)action.getValue("SmallIcon");
            }
        }
        this.setIcon(icon);
    }
    
    void smallIconChanged(Action iconFromAction) {
        if (iconFromAction.getValue("SwingLargeIconKey") == null) {
            this.setIconFromAction(iconFromAction);
        }
    }
    
    void largeIconChanged(Action iconFromAction) {
        this.setIconFromAction(iconFromAction);
    }
    
    private void setActionCommandFromAction(Action action) {
        this.setActionCommand((action != null) ? ((String)action.getValue("ActionCommandKey")) : null);
    }
    
    private void setSelectedFromAction(Action action) {
        boolean selected = false;
        if (action != null) {
            selected = AbstractAction.isSelected(action);
        }
        if (selected != (this.isSelected() ? 1 : 0)) {
            this.setSelected((boolean)selected);
            if (selected == 0 && this.isSelected() && this.getModel() instanceof DefaultButtonModel) {
                ButtonGroup group = ((DefaultButtonModel)this.getModel()).getGroup();
                if (group != null) {
                    group.clearSelection();
                }
            }
        }
    }
    
    protected PropertyChangeListener createActionPropertyChangeListener(Action action) {
        return this.createActionPropertyChangeListener0(action);
    }
    
    PropertyChangeListener createActionPropertyChangeListener0(Action action) {
        return new AbstractButton.ButtonActionPropertyChangeListener(this, action);
    }
    
    public boolean isBorderPainted() {
        return this.paintBorder;
    }
    
    public void setBorderPainted(boolean paintBorder) {
        int paintBorder2 = this.paintBorder;
        this.paintBorder = paintBorder;
        this.borderPaintedSet = true;
        this.firePropertyChange("borderPainted", (boolean)paintBorder2, this.paintBorder);
        if ((paintBorder ? 1 : 0) != paintBorder2) {
            this.revalidate();
            this.repaint();
        }
    }
    
    @Override
    protected void paintBorder(Graphics graphics) {
        if (this.isBorderPainted()) {
            super.paintBorder(graphics);
        }
    }
    
    public boolean isFocusPainted() {
        return this.paintFocus;
    }
    
    public void setFocusPainted(boolean paintFocus) {
        int paintFocus2 = this.paintFocus;
        this.firePropertyChange("focusPainted", (boolean)paintFocus2, this.paintFocus = paintFocus);
        if ((paintFocus ? 1 : 0) != paintFocus2 && this.isFocusOwner()) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public boolean isContentAreaFilled() {
        return this.contentAreaFilled;
    }
    
    public void setContentAreaFilled(boolean contentAreaFilled) {
        int contentAreaFilled2 = this.contentAreaFilled;
        this.contentAreaFilled = contentAreaFilled;
        this.contentAreaFilledSet = true;
        this.firePropertyChange("contentAreaFilled", (boolean)contentAreaFilled2, this.contentAreaFilled);
        if ((contentAreaFilled ? 1 : 0) != contentAreaFilled2) {
            this.repaint();
        }
    }
    
    public boolean isRolloverEnabled() {
        return this.rolloverEnabled;
    }
    
    public void setRolloverEnabled(boolean rolloverEnabled) {
        int rolloverEnabled2 = this.rolloverEnabled;
        this.rolloverEnabled = rolloverEnabled;
        this.rolloverEnabledSet = true;
        this.firePropertyChange("rolloverEnabled", (boolean)rolloverEnabled2, this.rolloverEnabled);
        if ((rolloverEnabled ? 1 : 0) != rolloverEnabled2) {
            this.repaint();
        }
    }
    
    public int getMnemonic() {
        return this.mnemonic;
    }
    
    public void setMnemonic(int mnemonic) {
        this.getMnemonic();
        this.model.setMnemonic(mnemonic);
        this.updateMnemonicProperties();
    }
    
    public void setMnemonic(char mnemonic) {
        if (mnemonic >= 'a' && mnemonic <= 'z') {
            mnemonic -= 32;
        }
        this.setMnemonic((int)mnemonic);
    }
    
    public void setDisplayedMnemonicIndex(int n) throws IllegalArgumentException {
        int mnemonicIndex = this.mnemonicIndex;
        if (n == -1) {
            this.mnemonicIndex = -1;
        }
        else {
            String text = this.getText();
            int n2 = (text == null) ? 0 : text.length();
            if (n < -1 || n >= n2) {
                throw new IllegalArgumentException("index == " + n);
            }
        }
        this.firePropertyChange("displayedMnemonicIndex", mnemonicIndex, this.mnemonicIndex = n);
        if (n != mnemonicIndex) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public int getDisplayedMnemonicIndex() {
        return this.mnemonicIndex;
    }
    
    private void updateDisplayedMnemonicIndex(String s, int n) {
        this.setDisplayedMnemonicIndex(SwingUtilities.findDisplayedMnemonicIndex(s, n));
    }
    
    private void updateMnemonicProperties() {
        int mnemonic = this.model.getMnemonic();
        if (this.mnemonic != mnemonic) {
            this.firePropertyChange("mnemonic", this.mnemonic, this.mnemonic = mnemonic);
            this.updateDisplayedMnemonicIndex(this.getText(), this.mnemonic);
            this.revalidate();
            this.repaint();
        }
    }
    
    public void setMultiClickThreshhold(long multiClickThreshhold) {
        if (multiClickThreshhold < 0L) {
            throw new IllegalArgumentException("threshhold must be >= 0");
        }
        this.multiClickThreshhold = multiClickThreshhold;
        return;
    }
    
    public long getMultiClickThreshhold() {
        return this.multiClickThreshhold;
    }
    
    public ButtonModel getModel() {
        return this.model;
    }
    
    public void setModel(ButtonModel buttonModel) {
        ButtonModel model = this.getModel();
        if (model != null) {
            model.removeChangeListener(this.changeListener);
            model.removeActionListener(this.actionListener);
            model.removeItemListener(this.itemListener);
            this.changeListener = null;
            this.actionListener = null;
            this.itemListener = null;
        }
        if ((this.model = buttonModel) != null) {
            this.changeListener = this.createChangeListener();
            this.actionListener = this.createActionListener();
            this.itemListener = this.createItemListener();
            buttonModel.addChangeListener(this.changeListener);
            buttonModel.addActionListener(this.actionListener);
            buttonModel.addItemListener(this.itemListener);
            this.updateMnemonicProperties();
            super.setEnabled(buttonModel.isEnabled());
        }
        else {
            this.mnemonic = 0;
        }
        this.updateDisplayedMnemonicIndex(this.getText(), this.mnemonic);
        this.firePropertyChange("model", model, buttonModel);
        if (buttonModel != model) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public ButtonUI getUI() {
        return (ButtonUI)this.ui;
    }
    
    public void setUI(ButtonUI ui) {
        super.setUI(ui);
        if ((this.disabledIcon instanceof UIResource)) {
            this.setDisabledIcon(null);
        }
        if ((this.disabledSelectedIcon instanceof UIResource)) {
            this.setDisabledSelectedIcon(null);
        }
    }
    
    @Override
    public void updateUI() {
    }
    
    @Override
    protected void addImpl(Component comp, Object constraints, int index) {
        if (!this.setLayout) {
            this.setLayout(new OverlayLayout(this));
        }
        super.addImpl(comp, constraints, index);
    }
    
    @Override
    public void setLayout(LayoutManager layout) {
        this.setLayout = true;
        super.setLayout(layout);
    }
    
    public void addChangeListener(ChangeListener l) {
        this.listenerList.add(ChangeListener.class, l);
    }
    
    public void removeChangeListener(ChangeListener l) {
        this.listenerList.remove(ChangeListener.class, l);
    }
    
    public ChangeListener[] getChangeListeners() {
        return (ChangeListener[])this.listenerList.getListeners(ChangeListener.class);
    }
    
    protected void fireStateChanged() {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ChangeListener.class) {
                if (this.changeEvent == null) {
                    this.changeEvent = new ChangeEvent(this);
                }
                ((ChangeListener)listenerList[i + 1]).stateChanged(this.changeEvent);
            }
        }
    }
    
    public void addActionListener(ActionListener l) {
        this.listenerList.add(ActionListener.class, l);
    }
    
    public void removeActionListener(ActionListener l) {
        if (l != null && this.getAction() == l) {
            this.setAction(null);
        }
        else {
            this.listenerList.remove(ActionListener.class, l);
        }
    }
    
    public ActionListener[] getActionListeners() {
        return (ActionListener[])this.listenerList.getListeners(ActionListener.class);
    }
    
    protected ChangeListener createChangeListener() {
        return this.getHandler();
    }
    
    protected void fireActionPerformed(ActionEvent actionEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ActionListener.class) {
                if (o == null) {
                    String command = actionEvent.getActionCommand();
                    if (command == null) {
                        command = this.getActionCommand();
                    }
                    o = new ActionEvent(this, 1001, command, actionEvent.getWhen(), actionEvent.getModifiers());
                }
                ((ActionListener)listenerList[i + 1]).actionPerformed(o);
            }
        }
    }
    
    protected void fireItemStateChanged(ItemEvent itemEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ItemListener.class) {
                if (o == null) {
                    o = new ItemEvent(this, 701, this, itemEvent.getStateChange());
                }
                ((ItemListener)listenerList[i + 1]).itemStateChanged(o);
            }
        }
        if (this.accessibleContext != null) {
            if (itemEvent.getStateChange() == 1) {
                this.accessibleContext.firePropertyChange("AccessibleState", null, AccessibleState.SELECTED);
                this.accessibleContext.firePropertyChange("AccessibleValue", Integer.valueOf(0), Integer.valueOf(1));
            }
            else {
                this.accessibleContext.firePropertyChange("AccessibleState", AccessibleState.SELECTED, null);
                this.accessibleContext.firePropertyChange("AccessibleValue", Integer.valueOf(1), Integer.valueOf(0));
            }
        }
    }
    
    protected ActionListener createActionListener() {
        return this.getHandler();
    }
    
    protected ItemListener createItemListener() {
        return this.getHandler();
    }
    
    @Override
    public void setEnabled(boolean b) {
        if (!b && this.model.isRollover()) {
            this.model.setRollover(false);
        }
        super.setEnabled(b);
        this.model.setEnabled(b);
    }
    
    @Deprecated
    public String getLabel() {
        return this.getText();
    }
    
    @Deprecated
    public void setLabel(String text) {
        this.setText(text);
    }
    
    @Override
    public void addItemListener(ItemListener l) {
        this.listenerList.add(ItemListener.class, l);
    }
    
    @Override
    public void removeItemListener(ItemListener l) {
        this.listenerList.remove(ItemListener.class, l);
    }
    
    public ItemListener[] getItemListeners() {
        return (ItemListener[])this.listenerList.getListeners(ItemListener.class);
    }
    
    @Override
    public Object[] getSelectedObjects() {
        if (!this.isSelected()) {
            return null;
        }
        return new Object[] { this.getText() };
    }
    
    protected void init(String text, Icon icon) {
        if (text != null) {
            this.setText(text);
        }
        if (icon != null) {
            this.setIcon(icon);
        }
        this.updateUI();
        this.setAlignmentX(0.0f);
        this.setAlignmentY(0.5f);
    }
    
    @Override
    public boolean imageUpdate(Image img, int infoflags, int x, int y, int w, int h) {
        Icon icon = null;
        if (!this.model.isEnabled()) {
            if (this.model.isSelected()) {
                icon = this.getDisabledSelectedIcon();
            }
            else {
                icon = this.getDisabledIcon();
            }
        }
        else if (this.model.isPressed() && this.model.isArmed()) {
            icon = this.getPressedIcon();
        }
        else if (this.isRolloverEnabled() && this.model.isRollover()) {
            if (this.model.isSelected()) {
                icon = this.getRolloverSelectedIcon();
            }
            else {
                icon = this.getRolloverIcon();
            }
        }
        else if (this.model.isSelected()) {
            icon = this.getSelectedIcon();
        }
        if (icon == null) {
            icon = this.getIcon();
        }
        return icon != null && SwingUtilities.doesIconReferenceImage(icon, img) && super.imageUpdate(img, infoflags, x, y, w, h);
    }
    
    @Override
    void setUIProperty(String s, Object o) {
        if (s == "borderPainted") {
            if (!this.borderPaintedSet) {
                this.setBorderPainted((boolean)o);
                this.borderPaintedSet = false;
            }
        }
        else if (s == "rolloverEnabled") {
            if (!this.rolloverEnabledSet) {
                this.setRolloverEnabled((boolean)o);
                this.rolloverEnabledSet = false;
            }
        }
        else if (s == "iconTextGap") {
            if (!this.iconTextGapSet) {
                this.setIconTextGap(((Number)o).intValue());
                this.iconTextGapSet = false;
            }
        }
        else if (s == "contentAreaFilled") {
            if (!this.contentAreaFilledSet) {
                this.setContentAreaFilled((boolean)o);
                this.contentAreaFilledSet = false;
            }
        }
        else {
            super.setUIProperty(s, o);
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",defaultIcon=" + ((this.defaultIcon != null && this.defaultIcon != this) ? this.defaultIcon.toString() : "") + ",disabledIcon=" + ((this.disabledIcon != null && this.disabledIcon != this) ? this.disabledIcon.toString() : "") + ",disabledSelectedIcon=" + ((this.disabledSelectedIcon != null && this.disabledSelectedIcon != this) ? this.disabledSelectedIcon.toString() : "") + ",margin=" + this.margin + ",paintBorder=" + (this.paintBorder ? "true" : "false") + ",paintFocus=" + (this.paintFocus ? "true" : "false") + ",pressedIcon=" + ((this.pressedIcon != null && this.pressedIcon != this) ? this.pressedIcon.toString() : "") + ",rolloverEnabled=" + (this.rolloverEnabled ? "true" : "false") + ",rolloverIcon=" + ((this.rolloverIcon != null && this.rolloverIcon != this) ? this.rolloverIcon.toString() : "") + ",rolloverSelectedIcon=" + ((this.rolloverSelectedIcon != null && this.rolloverSelectedIcon != this) ? this.rolloverSelectedIcon.toString() : "") + ",selectedIcon=" + ((this.selectedIcon != null && this.selectedIcon != this) ? this.selectedIcon.toString() : "") + ",text=" + this.text;
    }
    
    private AbstractButton.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new AbstractButton.Handler();
        }
        return this.handler;
    }
    
    static AbstractButton.Handler access$000(AbstractButton abstractButton) {
        return abstractButton.getHandler();
    }
    
    static void access$100(AbstractButton abstractButton) {
        abstractButton.updateMnemonicProperties();
    }
    
    private static class ButtonActionPropertyChangeListener extends ActionPropertyChangeListener<AbstractButton>
    {
        ButtonActionPropertyChangeListener(AbstractButton abstractButton, Action action) {
            super(abstractButton, action);
        }
        
        @Override
        protected void actionPropertyChanged(AbstractButton abstractButton, Action action, PropertyChangeEvent propertyChangeEvent) {
            if (AbstractAction.shouldReconfigure(propertyChangeEvent)) {
                abstractButton.configurePropertiesFromAction(action);
            }
            else {
                abstractButton.actionPropertyChanged(action, propertyChangeEvent.getPropertyName());
            }
        }
    }
    
    protected class ButtonChangeListener implements ChangeListener, Serializable
    {
        ButtonChangeListener() {
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            AbstractButton.access$000(AbstractButton.this).stateChanged(changeEvent);
        }
    }
    
    class Handler implements ActionListener, ChangeListener, ItemListener, Serializable
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            changeEvent.getSource();
            AbstractButton.access$100(AbstractButton.this);
            if (AbstractButton.this.isEnabled() != AbstractButton.this.model.isEnabled()) {
                AbstractButton.this.setEnabled(AbstractButton.this.model.isEnabled());
            }
            AbstractButton.this.fireStateChanged();
            AbstractButton.this.repaint();
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            AbstractButton.this.fireActionPerformed(actionEvent);
        }
        
        @Override
        public void itemStateChanged(ItemEvent itemEvent) {
            AbstractButton.this.fireItemStateChanged(itemEvent);
            if (AbstractButton.this.shouldUpdateSelectedStateFromAction()) {
                Action action = AbstractButton.this.getAction();
                if (action != null && AbstractAction.hasSelectedKey(action)) {
                    int selected = AbstractButton.this.isSelected();
                    if ((AbstractAction.isSelected(action) ? 1 : 0) != selected) {
                        action.putValue("SwingSelectedKey", Boolean.valueOf((boolean)selected));
                    }
                }
            }
        }
    }
    
    protected abstract class AccessibleAbstractButton extends JComponent.AccessibleJComponent implements AccessibleAction, AccessibleValue, AccessibleText, AccessibleExtendedComponent
    {
        @Override
        public String getAccessibleName() {
            String s = this.accessibleName;
            if (s == null) {
                s = (String)AbstractButton.this.getClientProperty("AccessibleName");
            }
            if (s == null) {
                s = AbstractButton.this.getText();
            }
            if (s == null) {
                s = super.getAccessibleName();
                return s;
            }
            return s;
        }
        
        @Override
        public AccessibleIcon[] getAccessibleIcon() {
            Icon icon = AbstractButton.this.getIcon();
            if ((icon instanceof Accessible)) {
                AccessibleContext accessibleContext = ((Accessible)icon).getAccessibleContext();
                if (accessibleContext != null && accessibleContext instanceof AccessibleIcon) {
                    return new AccessibleIcon[] { (AccessibleIcon)accessibleContext };
                }
            }
            return null;
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (AbstractButton.this.getModel().isArmed()) {
                accessibleStateSet.add(AccessibleState.ARMED);
            }
            if (AbstractButton.this.isFocusOwner()) {
                accessibleStateSet.add(AccessibleState.FOCUSED);
            }
            if (AbstractButton.this.getModel().isPressed()) {
                accessibleStateSet.add(AccessibleState.PRESSED);
            }
            if (AbstractButton.this.isSelected()) {
                accessibleStateSet.add(AccessibleState.CHECKED);
                return accessibleStateSet;
            }
            return accessibleStateSet;
        }
        
        @Override
        public AccessibleRelationSet getAccessibleRelationSet() {
            AccessibleRelationSet accessibleRelationSet = super.getAccessibleRelationSet();
            if (!accessibleRelationSet.contains(AccessibleRelation.MEMBER_OF)) {
                ButtonModel model = AbstractButton.this.getModel();
                if (model != null && model instanceof DefaultButtonModel) {
                    ButtonGroup group = ((DefaultButtonModel)model).getGroup();
                    if (group != null) {
                        int buttonCount = group.getButtonCount();
                        Object[] target = new Object[buttonCount];
                        Enumeration<AbstractButton> elements = group.getElements();
                        for (int i = 0; i < buttonCount; ++i) {
                            if (elements.hasMoreElements()) {
                                target[i] = elements.nextElement();
                            }
                        }
                        AccessibleRelation relation = new AccessibleRelation(AccessibleRelation.MEMBER_OF);
                        relation.setTarget(target);
                        accessibleRelationSet.add(relation);
                    }
                }
            }
            return accessibleRelationSet;
        }
        
        @Override
        public AccessibleAction getAccessibleAction() {
            return this;
        }
        
        @Override
        public AccessibleValue getAccessibleValue() {
            return this;
        }
        
        @Override
        public int getAccessibleActionCount() {
            return 1;
        }
        
        @Override
        public String getAccessibleActionDescription(int n) {
            if (n == 0) {
                return UIManager.getString("AbstractButton.clickText");
            }
            return null;
        }
        
        @Override
        public boolean doAccessibleAction(int n) {
            if (n == 0) {
                AbstractButton.this.doClick();
                return true;
            }
            return false;
        }
        
        @Override
        public Number getCurrentAccessibleValue() {
            if (AbstractButton.this.isSelected()) {
                return Integer.valueOf(1);
            }
            return Integer.valueOf(0);
        }
        
        @Override
        public boolean setCurrentAccessibleValue(Number n) {
            if (n == null) {
                return false;
            }
            if (n.intValue() == 0) {
                AbstractButton.this.setSelected(false);
            }
            else {
                AbstractButton.this.setSelected(true);
            }
            return true;
        }
        
        @Override
        public Number getMinimumAccessibleValue() {
            return Integer.valueOf(0);
        }
        
        @Override
        public Number getMaximumAccessibleValue() {
            return Integer.valueOf(1);
        }
        
        @Override
        public AccessibleText getAccessibleText() {
            if ((View)AbstractButton.this.getClientProperty("html") != null) {
                return this;
            }
            return null;
        }
        
        @Override
        public int getIndexAtPoint(Point point) {
            View view = (View)AbstractButton.this.getClientProperty("html");
            if (view != null) {
                Rectangle textRectangle = this.getTextRectangle();
                if (textRectangle == null) {
                    return -1;
                }
                return view.viewToModel((float)point.x, (float)point.y, new Rectangle2D.Float((float)textRectangle.x, (float)textRectangle.y, (float)textRectangle.width, (float)textRectangle.height), new Position.Bias[1]);
            }
            else {
                return -1;
            }
        }
        
        @Override
        public Rectangle getCharacterBounds(int n) {
            View view = (View)AbstractButton.this.getClientProperty("html");
            if (view != null) {
                Rectangle textRectangle = this.getTextRectangle();
                if (textRectangle == null) {
                    return null;
                }
                Rectangle2D.Float float1 = new Rectangle2D.Float((float)textRectangle.x, (float)textRectangle.y, (float)textRectangle.width, (float)textRectangle.height);
                try {
                    return view.modelToView(n, float1, Position.Bias.Forward).getBounds();
                }
                catch (BadLocationException ex) {
                    return null;
                }
            }
            else {
                return null;
            }
        }
        
        @Override
        public int getCharCount() {
            View view = (View)AbstractButton.this.getClientProperty("html");
            if (view != null) {
                Document document = view.getDocument();
                if ((document instanceof StyledDocument)) {
                    return ((StyledDocument)document).getLength();
                }
            }
            return AbstractButton.this.accessibleContext.getAccessibleName().length();
        }
        
        @Override
        public int getCaretPosition() {
            return -1;
        }
        
        @Override
        public String getAtIndex(int n, int n2) {
            if (n2 < 0 || n2 >= this.getCharCount()) {
                return null;
            }
            switch (n) {
                case 1: {
                    try {
                        return this.getText(n2, 1);
                    }
                    catch (BadLocationException ex) {
                        return null;
                    }
                }
                case 2: {
                    try {
                        String text = this.getText(0, this.getCharCount());
                        BreakIterator wordInstance = BreakIterator.getWordInstance(this.getLocale());
                        wordInstance.setText(text);
                        return text.substring(wordInstance.previous(), wordInstance.following(n2));
                    }
                    catch (BadLocationException ex2) {
                        return null;
                    }
                }
                case 3: {
                    try {
                        String text2 = this.getText(0, this.getCharCount());
                        BreakIterator sentenceInstance = BreakIterator.getSentenceInstance(this.getLocale());
                        sentenceInstance.setText(text2);
                        return text2.substring(sentenceInstance.previous(), sentenceInstance.following(n2));
                    }
                    catch (BadLocationException ex3) {
                        return null;
                    }
                }
                default: {
                    return null;
                }
            }
        }
        
        @Override
        public String getAfterIndex(int n, int n2) {
            if (n2 < 0 || n2 >= this.getCharCount()) {
                return null;
            }
            switch (n) {
                case 1: {
                    if (n2 + 1 >= this.getCharCount()) {
                        return null;
                    }
                    try {
                        return this.getText(n2 + 1, 1);
                    }
                    catch (BadLocationException ex) {
                        return null;
                    }
                }
                case 2: {
                    try {
                        String text = this.getText(0, this.getCharCount());
                        BreakIterator wordInstance = BreakIterator.getWordInstance(this.getLocale());
                        wordInstance.setText(text);
                        int following = wordInstance.following(n2);
                        if (following == -1 || following >= text.length()) {
                            return null;
                        }
                        int following2 = wordInstance.following(following);
                        if (following2 == -1 || following2 >= text.length()) {
                            return null;
                        }
                        return text.substring(following, following2);
                    }
                    catch (BadLocationException ex2) {
                        return null;
                    }
                }
                case 3: {
                    try {
                        String text2 = this.getText(0, this.getCharCount());
                        BreakIterator sentenceInstance = BreakIterator.getSentenceInstance(this.getLocale());
                        sentenceInstance.setText(text2);
                        int following3 = sentenceInstance.following(n2);
                        if (following3 == -1 || following3 > text2.length()) {
                            return null;
                        }
                        int following4 = sentenceInstance.following(following3);
                        if (following4 == -1 || following4 > text2.length()) {
                            return null;
                        }
                        return text2.substring(following3, following4);
                    }
                    catch (BadLocationException ex3) {
                        return null;
                    }
                }
                default: {
                    return null;
                }
            }
        }
        
        @Override
        public String getBeforeIndex(int n, int n2) {
            if (n2 < 0 || n2 > this.getCharCount() - 1) {
                return null;
            }
            switch (n) {
                case 1: {
                    if (n2 == 0) {
                        return null;
                    }
                    try {
                        return this.getText(n2 - 1, 1);
                    }
                    catch (BadLocationException ex) {
                        return null;
                    }
                }
                case 2: {
                    try {
                        String text = this.getText(0, this.getCharCount());
                        BreakIterator wordInstance = BreakIterator.getWordInstance(this.getLocale());
                        wordInstance.setText(text);
                        wordInstance.following(n2);
                        int previous = wordInstance.previous();
                        int previous2 = wordInstance.previous();
                        if (previous2 == -1) {
                            return null;
                        }
                        return text.substring(previous2, previous);
                    }
                    catch (BadLocationException ex2) {
                        return null;
                    }
                }
                case 3: {
                    try {
                        String text2 = this.getText(0, this.getCharCount());
                        BreakIterator sentenceInstance = BreakIterator.getSentenceInstance(this.getLocale());
                        sentenceInstance.setText(text2);
                        sentenceInstance.following(n2);
                        int previous3 = sentenceInstance.previous();
                        int previous4 = sentenceInstance.previous();
                        if (previous4 == -1) {
                            return null;
                        }
                        return text2.substring(previous4, previous3);
                    }
                    catch (BadLocationException ex3) {
                        return null;
                    }
                }
                default: {
                    return null;
                }
            }
        }
        
        @Override
        public AttributeSet getCharacterAttribute(int n) {
            View view = (View)AbstractButton.this.getClientProperty("html");
            if (view != null) {
                Document document = view.getDocument();
                if ((document instanceof StyledDocument)) {
                    Element characterElement = ((StyledDocument)document).getCharacterElement(n);
                    if (characterElement != null) {
                        return characterElement.getAttributes();
                    }
                }
            }
            return null;
        }
        
        @Override
        public int getSelectionStart() {
            return -1;
        }
        
        @Override
        public int getSelectionEnd() {
            return -1;
        }
        
        @Override
        public String getSelectedText() {
            return null;
        }
        
        private String getText(int n, int n2) throws BadLocationException {
            View view = (View)AbstractButton.this.getClientProperty("html");
            if (view != null) {
                Document document = view.getDocument();
                if ((document instanceof StyledDocument)) {
                    return ((StyledDocument)document).getText(n, n2);
                }
            }
            return null;
        }
        
        private Rectangle getTextRectangle() {
            String text = AbstractButton.this.getText();
            Icon icon = AbstractButton.this.isEnabled() ? AbstractButton.this.getIcon() : AbstractButton.this.getDisabledIcon();
            if (icon == null && text == null) {
                return null;
            }
            Rectangle rectangle = new Rectangle();
            Rectangle rectangle2 = new Rectangle();
            Rectangle rectangle3 = new Rectangle();
            Insets insets = AbstractButton.this.getInsets(new Insets(0, 0, 0, 0));
            rectangle3.x = insets.left;
            rectangle3.y = insets.top;
            rectangle3.width = AbstractButton.this.getWidth() - (insets.left + insets.right);
            rectangle3.height = AbstractButton.this.getHeight() - (insets.top + insets.bottom);
            SwingUtilities.layoutCompoundLabel(AbstractButton.this, this.getFontMetrics(this.getFont()), text, icon, AbstractButton.this.getVerticalAlignment(), AbstractButton.this.getHorizontalAlignment(), AbstractButton.this.getVerticalTextPosition(), AbstractButton.this.getHorizontalTextPosition(), rectangle3, rectangle, rectangle2, 0);
            return rectangle2;
        }
        
        @Override
        AccessibleExtendedComponent getAccessibleExtendedComponent() {
            return this;
        }
        
        @Override
        public String getToolTipText() {
            return AbstractButton.this.getToolTipText();
        }
        
        @Override
        public String getTitledBorderText() {
            return super.getTitledBorderText();
        }
        
        @Override
        public AccessibleKeyBinding getAccessibleKeyBinding() {
            int mnemonic = AbstractButton.this.getMnemonic();
            if (mnemonic == 0) {
                return null;
            }
            return new AbstractButton.AccessibleAbstractButton.ButtonKeyBinding(mnemonic);
        }
        
        class ButtonKeyBinding implements AccessibleKeyBinding
        {
            int mnemonic;
            
            ButtonKeyBinding(int mnemonic) {
                this.mnemonic = mnemonic;
            }
            
            @Override
            public int getAccessibleKeyBindingCount() {
                return 1;
            }
            
            @Override
            public Object getAccessibleKeyBinding(int n) {
                if (n != 0) {
                    throw new IllegalArgumentException();
                }
                return KeyStroke.getKeyStroke(this.mnemonic, 0);
            }
        }
    }
}
